#!/usr/bin/perl -w

# Copyright 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::KochCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','@dir4_to_dx','@dir4_to_dy',
  'PmOneBits','xy_rotate_plus90';
use MyPrintwrap ':all';


my $level = 3;

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::KochCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);


my ($x,$y) = (0,0);
my $start;
my $end;

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
my $join = '';
foreach my $n ($n_lo .. $n_hi-1) {
  my $dir4 = PmOneBits($n) % 4;
  my $dx = $dir4_to_dx[$dir4];
  my $dy = $dir4_to_dy[$dir4];
  my $x2 = $x + $dx;
  my $y2 = $y + $dy;
  my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

  my ($sdx,$sdy) = ($dx,$dy);
  my $sgap = '\\MyGap';
  if ($n >= 1 && $dir4 == (PmOneBits($n-1)+2) % 4) {
    my $xmid = $x . chamfer($dx,'\\MyEndMid');
    my $ymid = $y . chamfer($dy,'\\MyEndMid');
    printwrap("--($xmid,$ymid)");
    $sgap = '\\MyEndGap';
  }

  my ($edx,$edy) = ($dx,$dy);
  my $egap = '\\MyGap';
  if ($dir4 == (PmOneBits($n+1)+2) % 4) {
    $egap = '\\MyEndGap';
  }

  my $xoff = $x . chamfer($dx,$sgap) . chamfer($lx,'\\MyGap');
  my $yoff = $y . chamfer($dy,$sgap) . chamfer($ly,'\\MyGap');
  printwrap("$join($xoff,$yoff)");
  if ($n==$n_lo) { $start = "$xoff,$yoff"; }

  my $x2off = $x2 . chamfer(-$dx,$egap) . chamfer($lx,'\\MyGap');
  my $y2off = $y2 . chamfer(-$dy,$egap) . chamfer($ly,'\\MyGap');
  printwrap("--($x2off,$y2off)");
  if ($n==$n_hi-1) { $end = "$x2off,$y2off"; }

  $join = '--';
  ($x,$y) = ($x2,$y2);
}
printwrap(";\n");
print "  \\coordinate (start) at ($start);\n";
print "  \\coordinate (end) at ($end);\n";

print "  % end generated\n";
exit 0;
