#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','MfirstN','MlastN';
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternatePaper->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

$Printwrap = 2;
printwrap("\\draw[my line] (0,0)");
my $n_end = $n_hi+1;
foreach my $n ($n_lo+1 .. $n_end-1) {
  my ($x,$y) = $path->n_to_xy($n);

  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);
  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);
  printwrap("--($sx,$sy)");
  printwrap("--($tx,$ty)");
}
{
  my $n = $n_end;
  my ($x,$y) = $path->n_to_xy($n);
  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  printwrap("--($sx,$sy);\n");
}

$Printwrap = 2;
foreach my $s (0 .. 2**($level/2+1)-1) {
  my $from_n = MfirstN($s);
  my $to_n   = MlastN($s);
  my ($from_x,$from_y) = $path->n_to_xy($from_n);
  my ($to_x,$to_y) = $path->n_to_xy($to_n);
  if ($from_x == $to_x && $from_y == $to_y) {
    printwrap(" \\node at ($from_x,$from_y) [my dot];");
  } else {
    printwrap(" \\draw[->] ($from_x,$from_y) -- ($to_x,$to_y);");
  }
}
printwrap_newline();

print "  % end generated\n";
exit 0;
