#!/usr/bin/perl -w

# Copyright 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use Graph::Maker::TwinAlternateAreaTree;

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'TAVertexToZ_xy';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 8;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $n_hi = 2**$level - 1;
my $graph = Graph::Maker->new ('twin_alternate_area_tree',
                               level => $level,
                               undirected => 1);

foreach my $degree (1,2,3) {
  print "  \\begin{scope}[shift=(pos-$degree)]\n";

  my $segments = MyLineSegments->new;
  foreach my $edge ($graph->edges) {
    $segments->add(map {TAVertexToZ_xy($_)} @$edge);
  }

  $Printwrap = 4;
  printwrap("\\draw[my line] ");
  $Printwrap += 2;
  $segments->printwrap;
  printwrap(";\n");

  my $count = 0;
  $Printwrap = 4;
  printwrap("\\path foreach \\p in {");
  $Printwrap += 2;
  foreach my $n (sort $graph->vertices) {
    $graph->vertex_degree($n) == $degree or next;
    my ($x,$y) = TAVertexToZ_xy($n);
    printwrap("($x,$y),");
    $count++;
  }
  printwrap("}");
  printwrap(" { \\p node[my dot] };\n");
  print "% GP-Test-Last  TADegCount($level,$degree) == $count\n";
  print "  \\end{scope}\n";
}

print "  % end generated\n";
exit 0;
