#!/usr/bin/perl -w

# Copyright 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'dir','turn','@dir4_to_dx','@dir4_to_dy','chamfer';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 6;
if (@ARGV) { $level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternatePaper->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
### level: "$level  n_hi=$n_hi"

my $n_start  = ($level%2==0 ? 3 : 9) * $n_hi;
my $n_end    = $n_start + 4*$n_hi;
my $n_origin = $n_start + $n_hi;
my ($x_origin,$y_origin) = $path->n_to_xy($n_origin);
### origin: "$x_origin,$y_origin"

my $transform = sub {
  my ($x,$y) = @_;
  $x -= $x_origin;
  $y -= $y_origin;
  return ($x,$y);
};

{
  my $n = $n_start + $n_hi;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = $transform->($x,$y);
  print "  \\coordinate (start) at ($x,$y);\n";
}
{
  my $n = $n_start + 3*$n_hi;
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = $transform->($x,$y);
  print "  \\coordinate (end) at ($x,$y);\n";
  print "% GP-Test  Tend($level) == $x+I*$y\n";
}

my %seen;
my @nodes;
my $prev_txy;
my $offset = 3;  # after the first R to R segment
{
  my ($x,$y) = $path->n_to_xy($n_start+$offset-1);
  ($x,$y) = $transform->($x,$y);
  my ($dx,$dy) = $path->n_to_dxdy($n_start+$offset-1);
  my $tx = $x . chamfer($dx);
  my $ty = $y . chamfer($dy);
  $prev_txy = "$tx,$ty";
}
my ($x,$y) = $path->n_to_xy($n_start+$offset);
($x,$y) = $transform->($x,$y);
my $prev_dir = dir($n_start+$offset-1);
my $prev_turn = turn($n_start+$offset-1);
my $join = '';
my $count_segments = 0;
my $count_R = 0;
foreach my $n ($n_start+$offset .. $n_end-1,
               $n_start .. $n_start+$offset-1) {
  my $turn = turn($n);
  if ($n == $n_start) { $turn = -$turn; }
  my $dir = ($prev_dir + $turn) % 4;
  $count_segments++;

  my $prev_dx = $dir4_to_dx[$prev_dir];
  my $prev_dy = $dir4_to_dy[$prev_dir];

  my $dx = $dir4_to_dx[$dir];
  my $dy = $dir4_to_dy[$dir];

  my $sx = $x . chamfer(-$prev_dx);
  my $sy = $y . chamfer(-$prev_dy);
  my $tx = $x . chamfer($dx);
  my $ty = $y . chamfer($dy);

  if ($turn==-1 && ! $seen{"$x,$y"}) {
    push @nodes, "$x,$y";
  }

  if ($turn==-1 && $prev_turn==-1) {
    if ($join) {
      printwrap(";");
    }
    $Printwrap = 2;
    printwrap(" \\draw[my R line] ($prev_txy)--($sx,$sy);");
    $Printwrap += 2;
    $join = '';
    $count_R += ($turn == -1);
  }
  if (! $join) {
    $Printwrap = 2;
    printwrap(" \\draw[my L line]");
    $Printwrap += 2;
  }
  printwrap("$join($sx,$sy)");
  printwrap("--($tx,$ty)");
  $join = '--';

  $prev_turn = $turn;
  $prev_txy = "$tx,$ty";
  $prev_dir = $dir;
  $x += $dx;
  $y += $dy;
}
if ($join) {
  printwrap(";\n");
}

$Printwrap = 2;
printwrap("\\path foreach \\p in {");
$Printwrap += 2;
foreach my $node (@nodes) { printwrap("($node),"); }
printwrap("}");
printwrap(" { \\p node[my dot] };\n");

print "  % $count_segments segments, $count_R R turns\n";
print "  % end generated\n";
exit 0;
