#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use List::Util;
use Math::PlanePath::AlternatePaper 117; # v.117 for level_to_n_range()

use FindBin;
use lib $FindBin::Bin;
use MyAlternate 'chamfer','xy_rotate_plus90';
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $max_level = 5;  # in twinalternate numbering
if (@ARGV) { $max_level = shift @ARGV; }

print "  % generated by tools/$FindBin::Script\n";
my $path = Math::PlanePath::AlternatePaper->new;

foreach my $level (0 .. $max_level) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  ### level: "$level  n_hi=$n_hi"

  my $n_start  = ($level%2==0 ? 3 : 9) * $n_hi;
  my $n_end    = $n_start + 4*$n_hi;
  my $n_origin = $n_start + $n_hi;
  my ($x_origin,$y_origin) = $path->n_to_xy($n_origin);

  my $xyxy_is_twinalternate_touch = sub {
    my ($x,$y) = @_;
    my @n_list = $path->xy_to_n_list($x,$y);
    @n_list = grep {$n_start <= $_ && $_ < $n_end} @n_list;
    @n_list = map {int($_/$n_hi)} @n_list;
    @n_list = List::Util::uniq(@n_list);
    return @n_list == 2;
  };

  my $transform = sub {
    my ($x,$y) = @_;
    $x -= $x_origin;
    $y -= $y_origin;
    return ($x,$y);
  };

  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(pos-$level)]\n");
  $Printwrap += 2;

  for (my $n = $n_start; $n < $n_end; $n += $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    my $part = int( ($n-$n_origin) / $n_hi ) % 4;
    my $name = "p$part-$level";
    ($x,$y) = $transform->($x,$y);
    printwrap("\\node ($name) at ($x,$y) [my dot];\n");
  }

  my $join = '';
  printwrap("\\draw");
  $Printwrap += 2;
  foreach my $n ($n_start .. $n_end-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my $is_touch = $xyxy_is_twinalternate_touch->($x,$y);
    ($x,$y) = $transform->($x,$y);
    if ($is_touch) {
      my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
      my ($tdx,$tdy) = $path->n_to_dxdy($n);
      my $sx = $x . chamfer(-$sdx);
      my $sy = $y . chamfer(-$sdy);
      my $tx = $x . chamfer($tdx);
      my $ty = $y . chamfer($tdy);
      printwrap("$join($sx,$sy)");
      printwrap("--($tx,$ty)");
    } else {
      printwrap("$join($x,$y)");
    }
    $join = '--';
  }
  printwrap("--cycle;\n");
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
